import _classCallCheck from '@babel/runtime/helpers/esm/classCallCheck';
import _createClass from '@babel/runtime/helpers/esm/createClass';
import _get from '@babel/runtime/helpers/esm/get';
import _getPrototypeOf from '@babel/runtime/helpers/esm/getPrototypeOf';
import _inherits from '@babel/runtime/helpers/esm/inherits';
import _createSuper from '@babel/runtime/helpers/esm/createSuper';
import { getTextNodes, CARD_KEY, getHashId, isEngine, $, Tooltip, BlockPlugin } from '@aomao/engine';

var Outline = /*#__PURE__*/function () {
  function Outline() {
    _classCallCheck(this, Outline);

    this._invalidTags = {
      h1: true,
      h2: true,
      h3: true,
      h4: true,
      h5: true,
      h6: true
    };
  }

  _createClass(Outline, [{
    key: "_isHeadingNode",
    value: function _isHeadingNode(node) {
      if (node.nodeType !== 1) return false;
      if (!node.tagName) return false;
      var tag = node.tagName.toLocaleLowerCase();
      return this._invalidTags[tag] === true;
    }
    /**
     * 获取所有的 Heading 节点
     * @param {Node} rootNode
     * @private
     */

  }, {
    key: "_findHeadingNodes",
    value: function _findHeadingNodes(rootNode) {
      // querySelectorAll 会保证返回的标题列表按文档里面的顺序排序，但由于文档中列表、表格等中有标题，故只取一级节点
      // Ref: https://www.w3.org/TR/selectors-api/#queryselectorall
      // return Array.from(rootNode.querySelectorAll('h1,h2,h3,h4'));
      var nodes = rootNode.childNodes;
      if (!nodes) return [];
      var headings = [];

      for (var i = 0, l = nodes.length; i < l; i++) {
        var item = nodes[i];

        if (this._isHeadingNode(item)) {
          headings.push(item);
        }
      }

      return headings;
    } // 修订重复 id 的标题，借鉴 github 的模式：追加 -{出现顺序-1}

  }, {
    key: "_fixDuplicatedHeading",
    value: function _fixDuplicatedHeading(datas) {
      var map = {};
      datas.forEach(function (data) {
        var id = data.id;

        if (map.hasOwnProperty(id)) {
          var order = map[id];
          data.id += "-".concat(order); // 修订 domId，保证能准确定位

          if (data.domNode) data.domNode['id'] = data.id;
          map[id] = order + 1;
        } else {
          map[id] = 1;
        }
      });
    }
    /**
     * 根据标签名获取标题层级
     *
     * @param {string} tagName 标签名
     * @return {number}
     * @private
     */

  }, {
    key: "_getLevel",
    value: function _getLevel(tagName) {
      var map = {
        H1: 1,
        H2: 2,
        H3: 3,
        H4: 4,
        H5: 5,
        H6: 6
      };
      return map[tagName] || -1;
    }
  }, {
    key: "getText",
    value: function getText(element) {
      var nodes = getTextNodes(element, function (node) {
        return !!!node.getAttribute(CARD_KEY);
      });
      var text = '';
      nodes.forEach(function (node) {
        text += node.textContent;
      });
      return text;
    }
    /**
     * 将 heading 数据归一化为带深度层级的结构
     * 归一化后，每个元素的结构为:
     * {
     *   id: string,       // id
     *   title: string,    // 标题
     *   level: number ,   // 标题层级
     *   domNode: Node,    // dom 节点
     *   depth: number     // 展示深度
     * }
     * depth 的算法和 Google Docs 的一致
     * - 效果：h1 -> h4 分配固定的 depth，保证相同 level 的标题最终的层级深度是一样的
     * - 算法：找出文档存在的标题层级；按层级由大到小依次分别分配缩进深度；
     * @param {Element[]}headings heading 的标准 DOM 节点数组
     *
     * @return {Array} 节点数组
     */

  }, {
    key: "normalize",
    value: function normalize(headings) {
      var _this = this;

      headings = headings || [];
      if (headings.length === 0) return [];
      var data = [];
      headings.forEach(function (node) {
        var text = (_this.getText(node) || '').trim(); // id 或文本为空，不纳入大纲

        if (!text) return;
        var id = node['id'];

        if (!id) {
          id = node['data-id'] || getHashId(node);
        }

        data.push({
          id: id,
          text: text,
          // 层级
          level: _this._getLevel(node.tagName),
          // 深度
          depth: -1,
          // dom 节点，在后续处理中有可能用到，比如按需调整节点 id 等
          domNode: node
        });
      }); // 按 level 去重

      var numberList = [];
      data.forEach(function (heading) {
        if (numberList.indexOf(heading.level) < 0) {
          numberList.push(heading.level);
        }
      }); // 对出现的 level 进行排序

      numberList.sort(); // 构造 level -> depth 的 map

      var map = {};
      numberList.forEach(function (item, index) {
        map[item] = index + 1;
      }); // 追加 depth

      data.forEach(function (item) {
        item.depth = map[item.level];
      });
      return data;
    }
    /**
     * 从 DOM 节点提取大纲
     * @param {Element} rootNode
     * @return {Array}
     */

  }, {
    key: "extractFromDom",
    value: function extractFromDom(rootNode) {
      if (!rootNode) return [];

      try {
        var headings = this._findHeadingNodes(rootNode);

        var nodes = this.normalize(headings); // 修订重复标题

        this._fixDuplicatedHeading(nodes);

        return nodes;
      } catch (e) {
        return [];
      }
    }
  }]);

  return Outline;
}();

function styleInject(css, ref) {
  if ( ref === void 0 ) ref = {};
  var insertAt = ref.insertAt;

  if (!css || typeof document === 'undefined') { return; }

  var head = document.head || document.getElementsByTagName('head')[0];
  var style = document.createElement('style');
  style.type = 'text/css';

  if (insertAt === 'top') {
    if (head.firstChild) {
      head.insertBefore(style, head.firstChild);
    } else {
      head.appendChild(style);
    }
  } else {
    head.appendChild(style);
  }

  if (style.styleSheet) {
    style.styleSheet.cssText = css;
  } else {
    style.appendChild(document.createTextNode(css));
  }
}

var css_248z = ".data-anchor-button {\r\n\tposition: absolute;\r\n\twidth: 24px;\r\n\theight: 24px;\r\n\tline-height: 24px;\r\n\ttext-align: center;\r\n\tfont-size: 14px;\r\n\tdisplay: block;\r\n\tbackground: rgba(255, 255, 255, 0.9);\r\n\tcolor: #BFBFBF;\r\n\tborder-radius: 2px 2px;\r\n\tcursor: pointer;\r\n\t-webkit-user-select: none;\r\n\t-moz-user-select: none;\r\n\t-ms-user-select: none;\r\n\tuser-select: none;\r\n\topacity: 0;\r\n\tvisibility: hidden;\r\n\t-webkit-transition: opacity 0.3s ease-in-out;\r\n\ttransition: opacity 0.3s ease-in-out;\r\n}\r\n\r\n.data-anchor-button.data-anchor-button-active {\r\n\topacity: 1;\r\n\tvisibility: visible;\r\n}\r\n\r\n.data-anchor-button .data-icon {\r\n\twidth: 18px;\r\n}\r\n\r\n.data-anchor-button:hover {\r\n\tbackground: #f4f4f4;\r\n\tcolor: #595959;\r\n}\r\n\r\n.am-engine-view h1,.am-engine-view h2,.am-engine-view h3,.am-engine-view h4,.am-engine-view h5,.am-engine-view h6 {\r\n    position: relative;\r\n}\r\n\r\n.am-engine-view h1 .data-anchor-button,.am-engine-view h2 .data-anchor-button,.am-engine-view h3 .data-anchor-button,.am-engine-view h4 .data-anchor-button,.am-engine-view h5 .data-anchor-button,.am-engine-view h6 .data-anchor-button{\r\n    position: absolute;\r\n    left: -24px;\r\n    display: none;\r\n}\r\n\r\n.am-engine-view h1:hover .data-anchor-button,.am-engine-view h2:hover .data-anchor-button,.am-engine-view h3:hover .data-anchor-button,.am-engine-view h4:hover .data-anchor-button,.am-engine-view h5:hover .data-anchor-button,.am-engine-view h6:hover .data-anchor-button {\r\n    display: inline-block;\r\n    opacity: 1;\r\n    visibility: visible;\r\n}\r\n\r\n.am-engine-view h1 , .am-engine h1,.am-engine-view h2 , .am-engine h2,.am-engine-view h3 , .am-engine h3,.am-engine-view h4 , .am-engine h4,.am-engine-view h5 , .am-engine h5,.am-engine-view h6 , .am-engine h6 {\r\n    margin: 0;\r\n    word-spacing: 1px;\r\n    color: #262626;\r\n    font-weight: bold;\r\n    padding:0;\r\n}\r\n\r\n.am-engine-view h1 , .am-engine h1 {\r\n    font-size: 28px;\r\n    line-height: 36px;\r\n    padding: 7px 0;\r\n}\r\n\r\n.am-engine-view h2 , .am-engine h2 {\r\n    font-size: 24px;\r\n    line-height: 32px;\r\n    padding: 7px 0;\r\n}\r\n\r\n.am-engine-view h3 , .am-engine h3 {\r\n    font-size: 20px;\r\n    line-height: 28px;\r\n    padding: 7px 0;\r\n}\r\n\r\n.am-engine-view h4 , .am-engine h4 {\r\n    font-size: 16px;\r\n    line-height: 24px;\r\n    padding: 7px 0;\r\n}\r\n\r\n.am-engine-view h5 , .am-engine h5 {\r\n    font-size: 14px;\r\n    line-height: 24px;\r\n    padding: 7px 0;\r\n}\r\n\r\n.am-engine-view h6 , .am-engine h6 {\r\n    font-size: 14px;\r\n    line-height: 24px;\r\n    padding: 7px 0;\r\n    font-weight: normal;\r\n}";
styleInject(css_248z);

var _default = /*#__PURE__*/function (_BlockPlugin) {
  _inherits(_default, _BlockPlugin);

  var _super = _createSuper(_default);

  function _default() {
    var _this;

    _classCallCheck(this, _default);

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    _this = _super.call.apply(_super, [this].concat(args));
    _this.attributes = {
      id: '@var0'
    };
    _this.variable = {
      '@var0': /^[\w\.\-]+$/
    };
    _this.tagName = ['h1', 'h2', 'h3', 'h4', 'h5', 'h6'].filter(function (name) {
      return !_this.options.enableTypes || _this.options.enableTypes.indexOf(name) > -1;
    });
    _this.allowIn = ['blockquote', '$root'];
    _this.disableMark = _this.options.disableMark || ['fontsize', 'bold'];
    return _this;
  }

  _createClass(_default, [{
    key: "init",
    value: function init() {
      var _this2 = this;

      _get(_getPrototypeOf(_default.prototype), "init", this).call(this);

      var language = this.editor.language; //阅读模式处理

      if (!isEngine(this.editor) && this.options.showAnchor !== false) {
        this.editor.on('render', function (root) {
          var container = $(root);
          if (_this2.tagName.length === 0) return;
          container.find(_this2.tagName.join(',')).each(function (heading) {
            var node = $(heading);
            var id = node.attributes('id');

            if (id) {
              node.find('.data-anchor-button').remove();
              var button = $("<a class=\"data-anchor-button\"><span class=\"data-icon data-icon-".concat(node.name, "\"></span></a>"));

              if (node.height() !== 24) {
                button.css({
                  top: (node.height() - 24) / 2 + 'px'
                });
              }

              button.on('mouseenter', function () {
                Tooltip.show(button, language.get('copyAnchor', 'title').toString());
              });
              button.on('mouseleave', function () {
                Tooltip.hide();
              });
              button.on('click', function (e) {
                e.preventDefault();
                e.stopPropagation();
                var url = _this2.options.anchorCopy ? _this2.options.anchorCopy(id) : window.location.href + '/' + id;

                if (_this2.editor.clipboard.copy(url)) {
                  _this2.editor.messageSuccess(language.get('copy', 'success').toString());
                } else {
                  _this2.editor.messageError(language.get('copy', 'error').toString());
                }
              });
              node.prepend(button);
            }
          });
        });
      }

      if (isEngine(this.editor)) {
        this.editor.on('keydown:backspace', function (event) {
          return _this2.onBackspace(event);
        });
        this.editor.on('paste:markdown-check', function (child) {
          var _this2$checkMarkdown;

          return !((_this2$checkMarkdown = _this2.checkMarkdown(child)) === null || _this2$checkMarkdown === void 0 ? void 0 : _this2$checkMarkdown.match);
        });
        this.editor.on('paste:markdown', function (child) {
          return _this2.pasteMarkdown(child);
        });
        this.editor.on('paste:each', function (node) {
          if (_this2.tagName.indexOf(node.name) > -1) {
            node.attributes('id', '');
          }
        });
      } //引擎处理


      if (!isEngine(this.editor) || this.options.showAnchor === false) return;
      this.editor.on('setValue', function () {
        _this2.updateId();
      });
      this.editor.on('change', function () {
        _this2.updateId();

        _this2.showAnchor();
      });
      this.editor.on('select', function () {
        _this2.showAnchor();
      });
      window.addEventListener('resize', function () {
        _this2.updateAnchorPosition();
      }, false);
    }
  }, {
    key: "updateId",
    value: function updateId() {
      if (this.tagName.length === 0) return;
      this.editor.container.find(this.tagName.join(',')).each(function (titleNode) {
        var _node$parent;

        var node = $(titleNode);

        if (!((_node$parent = node.parent()) === null || _node$parent === void 0 ? void 0 : _node$parent.isEditable())) {
          node.removeAttributes('id');
          return;
        }

        var id = node.attributes('id');
        var dataId = node.attributes('data-id');

        if (!id || dataId !== id) {
          id = node.attributes('data-id') || getHashId(node);
          node.attributes('id', id);
        }
      });
    }
  }, {
    key: "updateAnchorPosition",
    value: function updateAnchorPosition() {
      var _root$get;

      if (!isEngine(this.editor)) return;
      var _this$editor = this.editor,
          change = _this$editor.change,
          root = _this$editor.root;
      var button = root.find('.data-anchor-button');

      if (button.length === 0) {
        return;
      }

      var range = change.getRange();
      var block = range.startNode.closest('h1,h2,h3,h4,h5,h6');

      if (block.length === 0) {
        button.remove();
        return;
      }

      var rootRect = ((_root$get = root.get()) === null || _root$get === void 0 ? void 0 : _root$get.getBoundingClientRect()) || {
        left: 0,
        top: 0
      };
      var rect = block.get().getBoundingClientRect();
      var left = Math.round(rect.left - rootRect.left - button.get().clientWidth - 1);
      var top = Math.round(rect.top - rootRect.top + rect.height / 2 - button.get().clientHeight / 2);
      button.css({
        top: "".concat(top, "px"),
        left: "".concat(left, "px")
      });
    }
  }, {
    key: "showAnchor",
    value: function showAnchor() {
      var _block$parent,
          _root$get2,
          _this3 = this;

      if (!isEngine(this.editor) || this.tagName.length === 0) return;
      var _this$editor2 = this.editor,
          change = _this$editor2.change,
          root = _this$editor2.root,
          clipboard = _this$editor2.clipboard,
          language = _this$editor2.language,
          card = _this$editor2.card;
      var range = change.getRange();
      var button = root.find('.data-anchor-button');
      var block = range.startNode.closest(this.tagName.join(','));

      if (block.length === 0 || button.length > 0 && button.find('.data-icon-'.concat(block.name)).length === 0) {
        button.remove();
      }

      if (block.length === 0 || card.closest(block, true)) {
        return;
      }

      if (!((_block$parent = block.parent()) === null || _block$parent === void 0 ? void 0 : _block$parent.isEditable())) {
        return;
      }

      if (button.find('.data-icon-'.concat(block.name)).length > 0) {
        this.updateAnchorPosition();
        return;
      }

      button = $("<span class=\"data-anchor-button\"><span class=\"data-icon data-icon-".concat(block.name, "\"></span></span>"));
      root.append(button);
      var parentRect = ((_root$get2 = root.get()) === null || _root$get2 === void 0 ? void 0 : _root$get2.getBoundingClientRect()) || {
        left: 0,
        top: 0
      };
      var rect = block.get().getBoundingClientRect();
      var left = Math.round(rect.left - parentRect.left - button.get().clientWidth - 1);
      var top = Math.round(rect.top - parentRect.top + rect.height / 2 - button.get().clientHeight / 2);
      button.css({
        top: "".concat(top, "px"),
        left: "".concat(left, "px")
      });
      button.addClass('data-anchor-button-active');
      button.on('mouseenter', function () {
        Tooltip.show(button, language.get('copyAnchor', 'title').toString());
      });
      button.on('mouseleave', function () {
        Tooltip.hide();
      });
      button.on('click', function (e) {
        e.preventDefault();
        e.stopPropagation();
        var id = block.attributes('id');
        var url = _this3.options.anchorCopy ? _this3.options.anchorCopy(id) : window.location.href + '/' + id;

        if (clipboard.copy(url)) {
          _this3.editor.messageSuccess(language.get('copy', 'success').toString());
        } else {
          _this3.editor.messageError(language.get('copy', 'error').toString());
        }
      });
    }
  }, {
    key: "execute",
    value: function execute(type) {
      if (!isEngine(this.editor)) return;
      if (!type || type === this.queryState()) type = 'p';
      var enableTypes = this.options.enableTypes; // 未启用

      if (type !== 'p' && enableTypes && enableTypes.indexOf(type) < 0) return;
      var _this$editor3 = this.editor,
          list = _this$editor3.list,
          block = _this$editor3.block;
      list.split();
      block.setBlocks("<".concat(type, " />"));
    }
  }, {
    key: "queryState",
    value: function queryState() {
      if (!isEngine(this.editor)) return;
      var change = this.editor.change;
      var blocks = change.blocks;

      if (blocks.length === 0) {
        return '';
      }

      var name = this.tagName.find(function (name) {
        return blocks.some(function (block) {
          return block.name === name;
        });
      });
      return name || '';
    }
  }, {
    key: "hotkey",
    value: function hotkey() {
      var _this$options$hotkey, _this$options$hotkey2, _this$options$hotkey3, _this$options$hotkey4, _this$options$hotkey5, _this$options$hotkey6;

      var h1Hotkey = ((_this$options$hotkey = this.options.hotkey) === null || _this$options$hotkey === void 0 ? void 0 : _this$options$hotkey.h1) || 'mod+opt+1';
      var h2Hotkey = ((_this$options$hotkey2 = this.options.hotkey) === null || _this$options$hotkey2 === void 0 ? void 0 : _this$options$hotkey2.h2) || 'mod+opt+2';
      var h3Hotkey = ((_this$options$hotkey3 = this.options.hotkey) === null || _this$options$hotkey3 === void 0 ? void 0 : _this$options$hotkey3.h3) || 'mod+opt+3';
      var h4Hotkey = ((_this$options$hotkey4 = this.options.hotkey) === null || _this$options$hotkey4 === void 0 ? void 0 : _this$options$hotkey4.h4) || 'mod+opt+4';
      var h5Hotkey = ((_this$options$hotkey5 = this.options.hotkey) === null || _this$options$hotkey5 === void 0 ? void 0 : _this$options$hotkey5.h5) || 'mod+opt+5';
      var h6Hotkey = ((_this$options$hotkey6 = this.options.hotkey) === null || _this$options$hotkey6 === void 0 ? void 0 : _this$options$hotkey6.h6) || 'mod+opt+6';
      var enableTypes = this.options.enableTypes;
      return [{
        key: h1Hotkey,
        args: 'h1'
      }, {
        key: h2Hotkey,
        args: 'h2'
      }, {
        key: h3Hotkey,
        args: 'h3'
      }, {
        key: h4Hotkey,
        args: 'h4'
      }, {
        key: h5Hotkey,
        args: 'h5'
      }, {
        key: h6Hotkey,
        args: 'h6'
      }].filter(function (item) {
        return !enableTypes || enableTypes.indexOf(item.key) > -1;
      });
    } //设置markdown

  }, {
    key: "markdown",
    value: function markdown(event, text, block) {
      if (!isEngine(this.editor) || this.options.markdown === false) return false;
      var type = '';

      switch (text) {
        case '#':
          type = 'h1';
          break;

        case '##':
          type = 'h2';
          break;

        case '###':
          type = 'h3';
          break;

        case '####':
          type = 'h4';
          break;

        case '#####':
          type = 'h5';
          break;

        case '######':
          type = 'h6';
          break;
      }

      if (!type) return true;
      var enableTypes = this.options.enableTypes; // 未启用

      if (enableTypes && enableTypes.indexOf(type) < 0) return true;
      event.preventDefault();
      this.editor.block.removeLeftText(block);

      if (this.editor.node.isEmpty(block)) {
        block.empty();
        block.append('<br />');
      }

      this.editor.command.execute(this.constructor.pluginName, type);
      return false;
    }
  }, {
    key: "checkMarkdown",
    value: function checkMarkdown(node) {
      if (!isEngine(this.editor) || !this.markdown || !node.isText()) return;
      var text = node.text();
      var reg = /(^|\r\n|\n)(#{1,6})(.*)/;
      var match = reg.exec(text);
      return {
        reg: reg,
        match: match
      };
    }
  }, {
    key: "pasteMarkdown",
    value: function pasteMarkdown(node) {
      var result = this.checkMarkdown(node);
      if (!result) return;
      var reg = result.reg,
          match = result.match;
      if (!match) return;
      var newText = '';
      var textNode = node.clone(true).get();
      var enableTypes = this.options.enableTypes;

      while (textNode.textContent && (match = reg.exec(textNode.textContent))) {
        var codeLength = match[2].length; //从匹配到的位置切断

        var regNode = textNode.splitText(match.index);
        newText += textNode.textContent; //从匹配结束位置分割

        textNode = regNode.splitText(match[0].length); // 过滤不支持的节点

        if (enableTypes && enableTypes.indexOf("h".concat(codeLength)) < 0) {
          newText += match[2] + match[3];
        } else newText += "<h".concat(codeLength, ">").concat(match[3].trim(), "</h").concat(codeLength, ">\n");
      }

      newText += textNode.textContent;
      node.text(newText);
    }
  }, {
    key: "onBackspace",
    value: function onBackspace(event) {
      var _block$parent2;

      if (!isEngine(this.editor)) return;
      var _this$editor4 = this.editor,
          change = _this$editor4.change,
          node = _this$editor4.node;
      var range = change.getRange();
      if (!range.collapsed) return;
      var blockApi = this.editor.block;
      if (!blockApi.isFirstOffset(range, 'start')) return;
      var block = blockApi.closest(range.startNode);

      if (this.tagName.indexOf(block.name) > -1 && node.isEmptyWithTrim(block) && ((_block$parent2 = block.parent()) === null || _block$parent2 === void 0 ? void 0 : _block$parent2.isEditable())) {
        event.preventDefault();
        blockApi.setBlocks('<p />');
        return false;
      }

      var parent = block.parent();

      if (this.tagName.indexOf(block.name) > -1 && (!parent || !node.isBlock(parent))) {
        event.preventDefault();
        change.mergeAfterDeletePrevNode(block);
        return false;
      }

      return;
    }
  }], [{
    key: "pluginName",
    get: function get() {
      return 'heading';
    }
  }]);

  return _default;
}(BlockPlugin);

export default _default;
export { Outline };
