export declare type OutlineData = {
    id: string;
    text: string;
    level: number;
    depth: number;
    domNode: Node;
};
declare class Outline {
    _invalidTags: {
        h1: boolean;
        h2: boolean;
        h3: boolean;
        h4: boolean;
        h5: boolean;
        h6: boolean;
    };
    _isHeadingNode(node: Element): boolean;
    /**
     * 获取所有的 Heading 节点
     * @param {Node} rootNode
     * @private
     */
    _findHeadingNodes(rootNode: Element): Array<Element>;
    _fixDuplicatedHeading(datas: Array<OutlineData>): void;
    /**
     * 根据标签名获取标题层级
     *
     * @param {string} tagName 标签名
     * @return {number}
     * @private
     */
    _getLevel(tagName: string): any;
    getText(element: Node): string;
    /**
     * 将 heading 数据归一化为带深度层级的结构
     * 归一化后，每个元素的结构为:
     * {
     *   id: string,       // id
     *   title: string,    // 标题
     *   level: number ,   // 标题层级
     *   domNode: Node,    // dom 节点
     *   depth: number     // 展示深度
     * }
     * depth 的算法和 Google Docs 的一致
     * - 效果：h1 -> h4 分配固定的 depth，保证相同 level 的标题最终的层级深度是一样的
     * - 算法：找出文档存在的标题层级；按层级由大到小依次分别分配缩进深度；
     * @param {Element[]}headings heading 的标准 DOM 节点数组
     *
     * @return {Array} 节点数组
     */
    normalize(headings: Array<Element>): OutlineData[];
    /**
     * 从 DOM 节点提取大纲
     * @param {Element} rootNode
     * @return {Array}
     */
    extractFromDom(rootNode: Element): OutlineData[];
}
export default Outline;
